/**
 * Videoplayer common superclass impelmentation for all inherited versions. Common interface that can be extended and specified
 * 
 * 
 *
 * @class VideoPlayerBasic
 * @constructor
 */
function VideoPlayerBasic(element_id, profile, width, height){
	console.log("VideoPlayerBasic - Constructor");
	this.FILETYPES = {
		MP4:0,
		MPEG:1,
		DASH:2
	};
	this.AVCOMPONENTS = {
		VIDEO : 0,
		AUDIO : 1,
		SUBTITLE : 2
	};
	
	this.currentItem = null; // set by menu.js:prepareVideoStart(), config.json item
	this.audioTrack = false;
	this.subtitleTrack = false;
	
	this.element_id = element_id;
	this.element = document.getElementById(element_id);
	if(!this.element){
		this.element = document.createElement("div");
		this.element.setAttribute("id", this.element_id);
	}
	$(this.element).addClass("hidden");
	//this.element.style.position="relative";
	this.fullscreenElement = this.element;
	this.width = width;
	this.height = height;
	this.visible = false;
	this.url = null;
	this.video = null;
	this.profile = profile;
	this.timeInMilliseconds = false;

	// Timers and intervals
	this.progressUpdateInterval = null;
	this.hidePlayerTimer = null;
	this.seekTimer = null;
	this.seekValue = 0;
	
	
	/**
	 * Creates player component and sets up event listeners
	 * Basic version is left empty and inherited players must define this method for all different players creation
	 * @method createPlayer
	**/
	this.createPlayer = this.__proto__.createPlayer;
	
	/**
	 * Basic video player populate method to initialize player html elements ready for setting up
	 * @method populate
	**/
	this.populate = function(){
		console.log("VideoPlayerBasic - populate");
		this.element.innerHTML = "";
		this.video = null;
		this.loadingImage = document.createElement("div");
		this.loadingImage.setAttribute("id", "loadingImage");
		this.loadingImage.addClass("hidden");
		this.element.appendChild(this.loadingImage);
		this.setFullscreen(true);
	};
	 	

	/**
	 * Displays player over video. Player shows current play position, duration and buttons that can be used
	 * @param {Int} sec. seconds player is displayed on screen and hidden after. If sec is not defined player remains on screen and is not hidden.
	 * @method displayPlayer
	 */
	this.displayPlayer = function( sec ){
		console.log("VideoPlayerBasic - displayPlayer");
		if(this.currentItem) {
			$("#player>#playText").text(this.currentItem.title);
			console.log("Current item=" + this.currentItem.title
				+"\n"+this.currentItem.url
				+"\ndrm="+this.currentItem.drm + " " + this.currentItem.la_url);
		} else {
			$("#player>#playText").text("");
		}
		clearTimeout( this.hidePlayerTimer );
		$("#player").removeClass("hide");
		$("#player").addClass("show");
		if(sec){
			this.hidePlayerTimer = setTimeout( function(){
				$("#player").removeClass("show");
			}, sec * 1000);
		}
	};

	this.getActiveButton = function() {
		if ( $("#prew").hasClass("activated") ) {
			return "#prew";
		} else if ( $("#pff").hasClass("activated") ) {
			return "#pff";
		} else if ( $("#ppauseplay").hasClass("activated") ) {
			return "#ppauseplay";
		}
	}

	this.nextButton = function(active, direction) {
		if (active == undefined) {
			return direction === 'left' ? "#prew" : "#pff";
		}

		if (active === "#prew") {
			return direction === 'left' ? "#prew" : "#ppauseplay";
		} else if (active === "#ppauseplay") {
			return direction === 'left' ? "#prew" : "#pff";
		} else if (active === "#pff") {
			return direction === 'left' ? '#ppauseplay' : '#pff';
		}
	}

	/**
	 * Handles navigation during video playback. This super class method may be re-defined on inherited class
	 * @param {Int} key. keycode of pressed key. Keycodes are defined in keycodes.js file
	 * @method navigate
	 */
	/* Use inherited basic method or player specified */
	this.navigate = this.__proto__.navigate || function(key){
		var self = this;
		
		if( dialog && dialog.open ){
			navigateDialog( key );
			return;
		}

		let active = this.getActiveButton();

		// this       = VideoPlayerEME, VideoPlayer(oipf) instance
		// this.video = HTMLVideoElement
		// this.player= DashJSPlayer instance
		switch(key){
			case VK_UP:
				self.displayPlayer(5);
			break;

			case VK_DOWN:
				self.displayPlayer(5);
			break;

			case VK_BACK:
			case VK_STOP:
			case 8: // for edge backspace button
				self.stop();
			break;

			case VK_LEFT:
				$("#prew, #pff, #ppauseplay").removeClass("activated");
				$(this.nextButton(active, 'left')).addClass("activated");
				self.displayPlayer(5);
				break;

			case VK_RIGHT:
				$("#prew, #pff, #ppauseplay").removeClass("activated");
				$(this.nextButton(active, 'right')).addClass("activated");
				self.displayPlayer(5);
				break;

			case VK_REWIND:
				self.seek( -30 );
				self.displayPlayer(5);
				break;

			case VK_FAST_FWD:
				self.seek( 30 );
				self.displayPlayer(5);
				break;

			case VK_ENTER:
				if (active === "#prew") {
					self.seek( -30 );
					self.displayPlayer(5);
				} else if (active === "#ppauseplay") {
					if( this.isPlaying() ){
						this.pause();
					}
					else{
						this.play();
					}
				} else if (active === "#pff") {
					self.seek( 30 );
					self.displayPlayer(5);
				}

				break;

			case VK_PLAY_PAUSE:
			case VK_PAUSE:
			case VK_PLAY:
				if( this.isPlaying() ){
					this.pause();
				}
				else{
					this.play();
				}
			break;

			case VK_YELLOW:
				try{
					if( this.video.textTracks ){
						var isEME=this.constructor.name=="VideoPlayerEME";
						
						// count all tracks except metadata
						var tracks = 0;
						var metadataTracks = [];
						var firstTextTrack = null;
						try{
							if(isEME) {
								tracks = this.getTextTracks(); // counter
								if(tracks>0) firstTextTrack=0;
							} else {							
								for( var i = 0; i < this.video.textTracks.length; ++i ){
									if( this.video.textTracks[i].kind != "metadata" ){
										if( firstTextTrack === null )
											firstTextTrack = i;
										tracks++;
										this.video.textTracks[i].mode = 'hidden'; // hide all
									} else {
										metadataTracks.push(i);
									}
								}
							}
						} catch(e){
							console.log("error " + e.description);
						}
						
						console.log("switch text track, tracks " + tracks);
						//console.log("metaDataTracks ", metadataTracks );
						if( !tracks ){
							showInfo("No Subtitles Available");
							break;
						}
						
						if( this.subtitleTrack === false )
							this.subtitleTrack = firstTextTrack;
						
						var lang;
						if(isEME) {	
							this.subtitleTrack = this.subtitleTrack >= tracks ? firstTextTrack : this.subtitleTrack+1;
							this.setTextTrack(this.subtitleTrack);
							lang = this.getCurrentTextTrack();
							if(lang=="undefined") lang="off";						
						} else {
							if( this.subtitleTrack >= tracks ){
								this.subtitleTrack = firstTextTrack; // current one was "off", select 1st track
							} else {
								this.video.textTracks[ this.subtitleTrack ].mode = 'hidden'; // hide current
								do{
									this.subtitleTrack++;
								} while( metadataTracks.indexOf( this.subtitleTrack ) != -1 );								
							}
							lang = (this.subtitleTrack >= tracks? "off" : this.video.textTracks[ this.subtitleTrack ].language );
							if(lang!="off")
								this.video.textTracks[ this.subtitleTrack ].mode = 'showing';
						}
						
						$("#subtitleButtonText").html( "Subtitles: " + lang );
						showInfo("Subtitles: " + lang);						
						if( lang != "off" )
							console.log("Set textTrack["+ this.subtitleTrack +"] Showing: " + lang);
					}
					else if( typeof this.enableSubtitles  == "function" ){
						this.changeAVcomponent( this.AVCOMPONENTS.SUBTITLE );
						//this.enableSubtitles(true);
					}
				} catch( e ){
					console.log( e.description );
				}
			break;
			
			case VK_GREEN:
				try{
					if( this.getAudioTracks() > 1 ){
						if( this.video.audioTracks ){
							console.log("switch audio track");
							if( this.audioTrack === false ) {
								this.audioTrack = 0;
							}

							var tracks = this.getAudioTracks();	 // counter
							if( this.audioTrack >= tracks ){
								this.audioTrack = 0; // was off(muted), select first and unmute audio
							} else {
								this.audioTrack++;
							}
							
							var lang;
							for (var i = 0; i < this.video.audioTracks.length; i += 1) {
								this.video.audioTracks[i].enabled = false;
							}
							var muted = ( this.audioTrack == tracks );
							if( !muted ){
								this.video.audioTracks[this.audioTrack].enabled = true;
							}
							lang = (muted? "Muted" : this.video.audioTracks[this.audioTrack].language );
							console.log("audiotracks " + tracks + ", current: "+this.audioTrack + ", " + lang);
							
							$("#audioButtonText").html( "Audio: " + lang );
							showInfo("Audio: " + lang);
						} else {
							this.changeAVcomponent( this.AVCOMPONENTS.AUDIO );
						}
					} else if( this.getAudioTracks() == 1 ) {
						showInfo("Current audio track (1/1): " + this.getCurrentAudioTrack() );
					} else {
						showInfo("No audio tracks available");
					}
				} catch( e ){
					console.log( e.description );
				}
			default:
			break;
		}
	};
	
	/**
	 * 
	 * @param {HTML Element} container. Container for video display. Video will be set inside the container 
	 * @method setDisplay
	 */
	this.setDisplay = function( container ){
		/*
		if( container ){
			// detach from DOM
			var element = $(this.element).detach();
			element.addClass("hidden");
			// append into
			$(container).prepend( element );
			element.removeClass("hidden");
		}
		else{
			// if target not set, assume to set fullscreen
			this.setFullscreen(true);
		}
		*/
	};
	
	this.getCurrentAudioTrack = this.__proto__.getCurrentAudioTrack || function(){
		return "default";
	};
	
	
	/**
	 * 
	 * @param {Object} subtitles. Creates HTML track objects for Out-Of-Band subtitle files
	 * @method setSubtitles
	 */
	this.setSubtitles = this.__proto__.setSubtitles || function( subtitles ){
		// out-of-band subtitles must be an array containing containing language code and source.xml file url.
		var self = this;
		try{
			var player = this.video;
			
			console.log("set subs from active assets metadata 'subtitles'");
			this.subtitles = subtitles;
			console.log("subtitles: "+JSON.stringify(this.subtitles) );
			
			if( this.subtitles && this.subtitles.length ){
				$.each( this.subtitles, function(i, lang){
					console.log("Subtitles " + i + ": " + lang.code + " - " + lang.src);
									
					var track = document.createElement("track");
					track.kind = "subtitles";
					track.label = lang.code;
					track.language = lang.code;
					// https://www.w3schools.com/tags/ref_language_codes.asp
					//if (lang.code=="eng") lang.isocode="en";
					//else if (lang.code=="fin") lang.isocode="fi";
					//else if (lang.code=="swe") lang.isocode="sv";
					//else if (lang.code=="ger") lang.isocode="de";
					track.srclang = lang.code; //lang.isocode;
					track.src = lang.src;
					track.onerror = function(e){
						self.lastError = e;
						console.log("track.onerror: "+JSON.stringify(e));
						//showInfo("Error getting subtitle file");
					};					
					player.appendChild(track);
				} );
				$("#subtitleButton").show();
				$("#subtitleButtonText").html( "Subtitles: " + player.textTracks[0].label );
				console.log( "Text tracks: " + player.textTracks.length );
				$.each( player.textTracks, function(i, track){
					console.log( track );
				} );
				this.subtitleTrack = 0;
				player.textTracks[0].mode = "showing";
			}
			else{
				console.log( "no subs" );
			}
		} catch(e){
			console.log("Error: setSubtitles: " + e.description );
		}
	};
	/**
	 * 
	 * Pause video playback and display player on screen
	 * @method pause
	 */
	this.pause = this.__proto__.pause || function(){
		console.log("VideoPlayerBasic pause");
		var self = this;
		try{
			self.video.pause();
			self.displayPlayer();
		}
		catch(e){
			console.log(e);
		}
	};
	
	/**
	 * 
	 * @param {bool} loading. Sets player's loading indicator visible if true, and hides if false
	 * @param {String} reason. logs reason for loading if specified
	 * @method setLoading
	 */
	this.setLoading = function(loading, reason){
		this.loading = loading;
		if(this.loading){
			this.loadingImage.removeClass("hidden");
		}
		else{
			this.loadingImage.addClass("hidden");
		}
		if(reason){
			console.log(reason);
		}
	};
	
	/**
	 * 
	 * @param {bool} fs. Set video fullscreen if true, and to the active asset box if false
	 * @method setFullscreen
	 */
	this.setFullscreen = function(fs){		
		this.fullscreen = fs;
		if(fs){
			this.element.addClass("fullscreen");
			//this.setDisplay( $("body")[0] ); // sets video player object to child of body
		}
		else{
			this.element.removeClass("fullscreen");
			//this.setDisplay( menu.focus.element ); // sets video player object to child of focused tile element
			$("#player").removeClass("show");
		}
	};
	
	/**
	 * 
	 * returns true if player is visible
	 * @method isVisible
	 */
	this.isVisible = function(fs){
		return this.visible;
	};
	
	/**
	 * 
	 * Resets progressbar and time to initial state
	 * @method resetProgressBar
	 */
	this.resetProgressBar = function(){
		try{
			var self = this;
			$("#progressbar")[0].style.width = "0px";
			$("#playposition").css("left", "0px");
			$("#progress_currentTime").css("left", "0px");
			$("#playposition").html( "00:00:00" );
			document.getElementById("playtime").innerHTML = self.live ? "LIVE" : "00:00:00";
		} catch(e){
			console.log( e.message );
		}
	};
	
	/**
	 * 
	 * Updates progress bar. Progress bar is only visible when player UI is displayed, but it is always updated non-visible when this method is called
	 * @method updateProgressBar
	 */
	this.updateProgressBar = function( sec ){
		var self = this;
		var position = 0;
		var duration = 0;
		var pbMaxWidth = 895; // progress bar maximum width in pixels
		
		var dtNow = new Date();
		var msElapsed = dtNow.getTime()-this.playbackStartTime;
		var sHMS  = getMillisAsHMS(msElapsed, true, true);
		$("#video-playtimer").text("Playback started " + sHMS + " ago");

		// if this was "setPersist(yes)" test then read persist_after status
		// see setDRM(), updateProgressBar()
		if(self.drm && msElapsed>5000 && self.drm.persist_before && !self.drm.persist_after) {
			self.drm.persist_after={}; // flag this request is done only once
			$.get(self.drm.persist_url, function(retval){
				self.drm.persist_after=retval;
				console.log("drm.persist_after " + JSON.stringify(retval));
				var befCount = self.drm.persist_before.status? self.drm.persist_before.status.requestCount || -1: -1;
				var aftCount = self.drm.persist_after.status ? self.drm.persist_after.status.requestCount || -1: -1;
				var tsLast   = self.drm.persist_after.status ? self.drm.persist_after.status.lastAccessedSince || -1 : -1;
				var info = "usePersist=" + (befCount!=aftCount ? "NO(laurl was invoked)" : "YES(reused a local license)")
					+ (tsLast>=0 ? "\nlastSince="+getMillisAsHMS(tsLast, true, true)+" ago" : "");
				console.log(info);
				showInfo(info, 7);
			}, "json");
		}
		
		// first get a time out of player and decide which player is used
		try{			
			if( this.live ){
				duration = 100;
				position = 100;
				var time = this.time();				
				duration = time.duration;
				position = time.position;
			}
			else{
				// <video> object used
				var time = this.time();
				if(this.video && this.video.duration ){
					position = (sec ? sec + this.video.currentTime : this.video.currentTime);
					duration = this.video.duration;
				}
				// oipf player object used. Convert milliseconds to seconds
				else if(this.video && this.video.playTime ){
					position = (sec? this.video.playPosition / 1000 + sec : this.video.playPosition / 1000);
					duration = this.video.playTime / 1000;
				} else {
					//console.log("Videoplayer not ready. Can not get position or duration");
					return;
				}
			}
		} catch(e){
			console.log( e.message );
		}
		
		try{
			pbar = document.getElementById("progressbar");

			var barWidth=0;
			if(duration!=0) barWidth = Math.floor((position / duration) * pbMaxWidth );
			if(barWidth > pbMaxWidth){
				barWidth = pbMaxWidth;
			}
			else if( barWidth < 0 ){
				barWidth = 0;
			}
			
			pbar.style.width = barWidth + "px";
			
			var play_position = barWidth;
			
			//console.log(  play_position, position, duration );
			
			$("#playposition").css("left", play_position);
			$("#progress_currentTime").css("left", play_position);

			$("#playposition").html("");
			if(position){
				$("#playposition").html(getMillisAsHMS(position*1000));
			}

			document.getElementById("playtime").innerHTML = "";
			if(duration){
				if( duration == Infinity || self.live ){
					document.getElementById("playtime").innerHTML = "LIVE";
				}
				else{
					document.getElementById("playtime").innerHTML = getMillisAsHMS(duration*1000);
				}
			}
		} catch(e){
			console.log( e.message );
		}

	};
	
	/**
	 * Initialize drm values, may trigger "setPersist(yes)" counter.
	 * @param {String} system. Specifies DRM system to be used as a string value (for example playready or marlin)
	 * @param {String} la_url. DRM lisence url
	 * @method setDRM
	 */
	this.setDRM = function(system, la_url){
		if( !system ){
			this.drm = null;
		} else {
			console.log("setDRM("+ system +", "+la_url+")");
			this.drm = { la_url : la_url, system : system, ready : false, error : null
				, persist_before:null, persist_after:null, persist_url:null
			};

			// keep track of persistence request count before and after playback.
			// see setDRM(), updateProgressBar()
			var delimS = this.currentItem.desc.indexOf("setPersist(");
			var delimE = delimS>=0 ? this.currentItem.desc.indexOf(")", delimS) : -1;
			var sValue = delimE>=0 ? this.currentItem.desc.substring(delimS,delimE+1) : "";
			if(sValue=="setPersist(yes)") {
				var self=this;
				self.drm.persist_url = la_url.replace(/\?persist\=1\&/, "?persist=status&"); // "?persist=1&" to "?persist=status&"
				$.get(self.drm.persist_url, function(retval){
					console.log("drm.persist_before " + JSON.stringify(retval));
					self.drm.persist_before=retval;
				}, "json");
			} // else "setPersist(no)" or ""
		}
	};

	/**
	 * Return players playback position and duration as an object with duration and position value.
	 * Times are represented in seconds for all players
	 * @method time
	 * @returns (Object) { duration : \d+, position : \d+ }
	 */
	this.time = function(){
		try{			
			if( this.timeInMilliseconds && this.video.playTime ){
				return { duration : this.video.playTime/1000, position : this.video.playPosition/1000 };
			}
			else if( this.video.duration ){
				return { duration : this.video.duration, position : this.video.currentTime };
			}
			else{
				//console.log("timedata not available")
				return { duration : 0, position : 0 }; // timedata not available
			}
			
		} catch(e){
			//console.log("error getting playback position and duration");
			return { duration : 0, position : 0 };
		}
		
	}
	
	/**
	 * Perform seek operation. To be called when user presses seek button. 
	 * Timeout is set to wait for continious seek operations before actual seeking
	 * If this is called multiple times the value to be seeked is added up.
	 * After delay of 700ms the seek is performed
	 * @method seek
	 * @param (Int) sec: How many seconds is seeked. Positiove integer for forward, negative for rewind.
	 */
	this.seek = function( sec ){
		//console.log("seek: " + sec);
		var self = this;
		try{			
			//console.log( this.time(), this.seekValue, sec );
			// if seek value goes below zero seconds, do immediate seek
			if( this.time().position + (this.seekValue + sec) < 0 ){
				console.log( "seek below starting position. go to start" );
				clearTimeout( this.seekTimer );
				this.updateProgressBar( -this.time().position );
				self.video.seek( -( self.timeInMilliseconds? this.time().position * 1000 : this.time().position ) );
				self.seekValue = 0;
				clearTimeout( this.seekTimer );
				self.seekTimer = null;
				return;
			}
			
			// if seek value goes above playtime, do not add seek seconds
			if( this.time().position + (this.seekValue + sec) > this.time().duration ){
				return;
			}
			
			this.seekValue += sec;
			
			console.log("seek video "+ this.seekValue +"s");
			this.updateProgressBar( self.seekValue );
			clearTimeout( this.seekTimer );
			
			this.seekTimer = setTimeout( function(){
				//console.log("perform seek now!");
				self.seekTimer = null;
				try{
					// if oipf player, form toSeek value absolute
					var toSeek = (self.timeInMilliseconds? ( self.video.playPosition + (self.seekValue * 1000) ) : self.seekValue);					
					//console.log("seekValue: " + self.seekValue);
					self.video.seek( toSeek );
					Monitor.videoSeek( self.seekValue );
					console.log("seek completed to " + toSeek);
				} catch(e){
					console.log("seek failed: " + e.description);
				}
				
				self.seekValue = 0;
			}, 700);

			/* TODO
			var buttonActivated = ( sec < 0? "prew" : "pff" );
			$("#prew, #pff" ).removeClass("activated");
			$("#" + buttonActivated ).addClass("activated");

			clearTimeout( this.seekActiveTimer );
			this.seekActiveTimer = setTimeout( function(){
				$("#prew, #pff" ).removeClass("activated");
			}, 700);
			 */
		}
		catch(e){
			console.log(e.message);
			console.log(e.description);
		}
	};

	
	/******************************
	MEMORY / Watched playpositions to cookies

	watched object shall hold up to 5 play positions of the watched videos (latest one on each latest series).

	watched.list : list of watched assets holding last play position (playtime) and timestamp (ts) when updated
	watched.set( time, duration ) : sets play position for video (with unique id).
	watched.get( id ) : gets play position and timestamp of a video (id), eg. { playtime : 12000, ts : 1438337491707 }
	watched.save() : saves list to cookie
    watched.delete() : delete watched cookie
	
	NOT USED ANYMORE, functions do nothing.
	*******************************/
	this.watched = {
		list : [],
		current : null,
		
		set : function(time, duration, videoid){ return; },
		save: function(){ },
		get : function(id){ return null; },
		load: function(successCB){ },
		deleteCurrent: function(){ }
		
		/*set : function( time, duration, videoid ){			
			// do not save if watched less than 10s
			if( time < 10 )
				return;
			
			// drop data if watched near to end
			if( time > duration - 15 ){
				if( this.current !== null ){
					this.deleteCurrent();
					 this.current = null;
					 //console.log("deleted record for video " + videoid);
				}
				//console.log("play positio not saved. too close to end");
				return;
			}
			
			var item = null;
			if( this.current === null && videoid ){
				//console.log("create watched new item");
				item = { videoid : videoid, position : time, duration : duration };
			}
			else if( this.current !== null ){
				//console.log("update playposition for ", this.list[ this.current ]);
				this.list[ this.current ].position = time;
			} else {
				console.log( "videoid is missing" );
				return;
			}
			
			// new item was not before in the list
			if( item ){
				//console.log("new item to first slot of the list");
				this.list.unshift( item );
				this.current = 0;
				// if list is full drop off last
				if( this.list.length > 10 ){
					this.list.pop();
				}
			}
			
		},
		save : function()
		{
			expiry = Math.round( (new Date()).getTime() + 1000 * 60 * 60 *24 * 30 );
			createCookie( "RefappWatched", JSON.stringify( this.list ), expiry );
		},
		get : function( id )
		{
			var found = null;
			var self = this;
			self.current = null;
			$.each( self.list, function(index, value){
				if( value.videoid == id ){
					found = value;
					self.list.splice(index, 1);
					console.log("found previously watched item ", found, "removed from list", self.list);
					// set to first
					self.list.unshift( value );
					console.log("and added to first item ", self.list );
					self.current = 0; // current is first
					return false;
				}
			} );
			
			return found;
		},
		load : function( successCB ){
			
			try{	
				this.list = JSON.parse( readCookie("RefappWatched") ) || [];
			}
			catch(e){
				console.log( "error: " + e.message );
				// corrupted cookie
				this.list = [];

			}
		},
		deleteCurrent: function() {
			this.list.splice(this.current, 1);
		}*/
	};
	
}
